# -*- coding: utf-8 -*-
# pylint: disable=missing-docstring
from __future__ import absolute_import, unicode_literals

# stdlib imports
from unittest import TestCase

# local imports
from serafin import Context
from serafin.core import dump_val
from serafin.serializers import serialize_iterable
from serafin.fieldspec import Fieldspec


class TestSerializeIterable(TestCase):
    def test_only_primitives_inside_one_level_deep(self):
        data = [1, 'test', 3.14159]
        ctx = Context(dumpval=dump_val)
        out = serialize_iterable(data, Fieldspec('*'), ctx)
        self.assertListEqual(out, data)

    def test_converts_string_to_Fieldspec(self):
        data = [1, 'test', 3.14159]
        ctx = Context(dumpval=dump_val)
        out = serialize_iterable(data, Fieldspec('*'), ctx)
        self.assertListEqual(out, data)



# -*- coding: utf-8 -*-
# pylint: disable=missing-docstring
from __future__ import absolute_import, unicode_literals

# stdlib imports
import unittest
from functools import partial

# local imports
from serafin import Context, serialize


class CustomClass(Context):
    pass


class TestSerialize(unittest.TestCase):
    def _test_dict(self):
        return {
            'field1': 10,
            'field2': {
                'sub1': 1,
                'sub2': 2,
            },
            'field3': 20,
        }

    def test_serialize__all(self):
        model = self._test_dict()
        self.assertDictEqual(serialize(model, '*'), {
            'field1': 10,
            'field2': {},
            'field3': 20
        })

    def test_serialize__selected(self):
        model = self._test_dict()

        self.assertDictEqual(serialize(model, 'field1,field3'), {
            'field1': 10,
            'field3': 20
        })

    def test_serialize__expand_selected(self):
        model = self._test_dict()
        self.assertDictEqual(serialize(model, 'field1,field2(sub1)'), {
            'field1': 10,
            'field2': {
                'sub1': 1
            }
        })

    def test_serialize__expand_all(self):
        model = self._test_dict()
        self.assertDictEqual(serialize(model, 'field1,field2(*)'), {
            'field1': 10,
            'field2': {
                'sub1': 1,
                'sub2': 2
            }
        })

    def test_serialize__all_recursive(self):
        model = self._test_dict()

        self.assertDictEqual(serialize(model, '**'), {
            'field1': 10,
            'field2': {
                'sub1': 1,
                'sub2': 2
            },
            'field3': 20
        })

    def test_context_is_being_passed(self):
        # pylint: disable=unused-variable
        @serialize.type(CustomClass)
        def custom_serialize(obj, spec, ctx):
            ctx.called()
            self.assertIn('test_val', ctx)
            self.assertEqual(ctx.test_val, 123)

        tmp = Context(called=False)
        obj = CustomClass(test_val=321)
        called = partial(setattr, tmp, 'called', True)
        serialize(obj, '*', test_val=123, called=called)
        self.assertTrue(tmp.called)



# -*- coding: utf-8 -*-
# pylint: disable=missing-docstring
from __future__ import absolute_import, unicode_literals

# stdlib imports
from unittest import TestCase

# local imports
from serafin import Context
from serafin.core import dump_val
from serafin.serializers import serialize_dict
from serafin.fieldspec import Fieldspec


class TestSerializeDict(TestCase):
    def test_only_primitives_inside_one_level_deep(self):
        data = {
            'field1':  123,
            'field2':  "string",
            'field3':  123.321,
            'field4':  {
                'field1': 3.14159,
                'field2': 'sub val'
            }
        }
        ctx = Context(dumpval=dump_val)
        result = serialize_dict(data, Fieldspec('*'), ctx)
        self.assertDictEqual(result, {
            'field1':  123,
            'field2':  "string",
            'field3':  123.321,
            'field4':  {}
        })

    def test_converts_string_to_fieldspec(self):
        ctx = Context(dumpval=dump_val)
        output = serialize_dict({'test': 123}, Fieldspec('*'), ctx)
        self.assertDictEqual(output, {'test': 123})



# -*- coding: utf-8 -*-
# pylint: disable=missing-docstring
from __future__ import absolute_import, unicode_literals

# stdlib imports
from unittest import TestCase
from datetime import datetime

# local imports
from serafin import Context
from serafin.core import dump_val
from serafin.serializers import serialize_primitive
from serafin.fieldspec import Fieldspec


class TestPrimitives(TestCase):
    def test_serialize_int(self):
        ctx = Context(dumpval=dump_val)
        out = serialize_primitive(5, Fieldspec('*'), ctx)
        self.assertEqual(out, 5)

    def test_serialize_None(self):
        ctx = Context(dumpval=dump_val)
        out = serialize_primitive(None, Fieldspec('*'), ctx)
        self.assertEqual(out, None)

    def test_serialize_str(self):
        ctx = Context(dumpval=dump_val)
        out = serialize_primitive('test str', Fieldspec('*'), ctx)
        self.assertEqual(out, 'test str')

    def test_serialize_datetime(self):
        dt = datetime(2015, 5, 1, 13, 21, 38)
        ctx = Context(dumpval=dump_val)
        out = serialize_primitive(dt, Fieldspec('*'), ctx)
        self.assertEqual(out, dt.isoformat())



# -*- coding: utf-8 -*-
# pylint: disable=missing-docstring
from __future__ import absolute_import, unicode_literals

# 3rd party imports
# import pytest
# # import pytz
# # from django.db import models as db
# # from factory import fuzzy
# # from factory.django import DjangoModelFactory
#
# # local imports
# # from serafin import serialize
# # enable model serializer           pylint: disable=unused-import
# from serafin.django import serialize_django_model
#
#
# # class SimpleModel(db.Model):
# #     """ A simple model without any relations. """
# #     name = db.CharField(max_length=100)
# #     age = db.IntegerField()
# #     height = db.FloatField()
# #     created_dt = db.DateTimeField(auto_now_add=True)
# #
# #     @property
# #     def days_exists(self):
# #         """ Return the number of days since the model was created. """
# #         diff = datetime.now(pytz.utc) - self.created_dt
# #         return diff.total_seconds() / 86400
# #
# #
# # class SimpleModelFactory(DjangoModelFactory):
# #     """ Easily create SimpleModel instances. """
# #     class Meta:     # pylint: disable=missing-docstring
# #         model = SimpleModel
# #
# #     name = fuzzy.FuzzyText(length=24)
# #     age = fuzzy.FuzzyInteger(10, 40)
# #     height = fuzzy.FuzzyFloat(1.3, 2.3)
# #     created_dt = fuzzy.FuzzyDateTime(datetime(2017, 10, 1, tzinfo=pytz.utc))
# #
# #
# # class Person(db.Model):
# #     """ Fake person model for testing. """
# #     name = db.CharField(max_length=100)
# #     age = db.IntegerField()
# #     height = db.FloatField()
# #     created_dt = db.DateTimeField(auto_now_add=True)
# #
# #     @property
# #     def days_active(self):
# #         """ Days since the person record was created. """
# #         diff = datetime.now(pytz.utc) - self.created_dt
# #         return diff.total_seconds() / 86400
# #
# #
# # class PersonFactory(DjangoModelFactory):
# #     """ Easily create Person instances. """
# #     class Meta:     # pylint: disable=missing-docstring
# #         model = Person
# #
# #     name = fuzzy.FuzzyText(length=24)
# #     age = fuzzy.FuzzyInteger(10, 40)
# #     height = fuzzy.FuzzyFloat(1.3, 2.3)
# #     created_dt = fuzzy.FuzzyDateTime(datetime(2017, 10, 1, tzinfo=pytz.utc))
# #
# #
# # class BlogPost(db.Model):
# #     """ Fake blog post model for testing. """
# #     title = db.CharField(max_length=200)
# #     content = db.TextField()
# #     author = db.ForeignKey('Person', related_name='blog_posts')
# #
# #
# # class BlogPostFactory(DjangoModelFactory):
# #     """ Easily create BlogPost instances. """
# #     class Meta:     # pylint: disable=missing-docstring
# #         model = BlogPost
# #
# #     title = fuzzy.FuzzyText(length=24)
# #     content = fuzzy.FuzzyText(length=400)
# #     author = SubFactory(PersonFactory)
# #
#
# # ########### Tests #############
#
#
# @pytest.mark.skip()
# @pytest.mark.django_db
# def test_serializes_model_fields():
#     model = SimpleModelFactory.build()
#
#     data = serialize(model, '*')
#
#     assert 'name' in data
#
#
# @pytest.mark.skip()
# @pytest.mark.django_db
# def test_serializes_properties():
#     model = SimpleModelFactory.build()
#
#     data = serialize(model, '*')
#
#     assert 'days_exists' in data
#
#
# @pytest.mark.skip()
# @pytest.mark.django_db
# def test_can_get_a_single_field():
#     model = SimpleModelFactory.build()
#
#     data = serialize(model, 'name')
#
#     assert frozenset(data.keys()) == frozenset((
#         'name',
#     ))
#
#
# @pytest.mark.skip()
# @pytest.mark.django_db
# def test_doesnt_serialize_related_if_not_expanded_in_fieldspec():
#     person = PersonFactory()
#     BlogPostFactory(author=person)
#     BlogPostFactory(author=person)
#     BlogPostFactory(author=person)
#
#     data = serialize(person, '*')
#
#     assert 'blog_posts' in data
#     assert len(data['blog_posts']) == 0
# #
# #
# # @pytest.mark.django_db
# # def test_serializes_foreign_keys_as_empty_objects_if_not_expanded():
# #     column = ColumnFactory()
# #
# #     data = serialize(column, 'table')
# #
# #     assert 'table' in data
# #     assert data['table'] == {}
# #
# #
# # @pytest.mark.django_db
# # def test_serializes_foreign_keys_properly_when_expanded():
# #     column = ColumnFactory()
# #
# #     data = serialize(column, 'table(name)')
# #
# #     assert 'table' in data
# #     assert 'name' in data['table']
#
#
# # @pytest.mark.django_db
# # def test_serializes_related_if_expanded_in_fieldspec():
# #     table = SimpleModelFactory()
# #     ColumnFactory(table=table)
# #     ColumnFactory(table=table)
# #     ColumnFactory(table=table)
# #
# #     data = serialize(table, '*,columns(*)')
# #
# #     assert 'columns' in data
# #     assert len(data['columns']) == 3
# #     assert 'name' in data['columns'][0]
# #     assert 'type' in data['columns'][0]
# #     assert 'table' in data['columns'][0]
#
#
# @pytest.mark.skip()
# @pytest.mark.django_db
# def test_serializes_only_specified_related_fields():
#     person = PersonFactory()
#     BlogPostFactory(author=person)
#     BlogPostFactory(author=person)
#     BlogPostFactory(author=person)
#
#     data = serialize(person, '*,blog_posts(title)')
#
#     assert 'blog_posts' in data
#     assert len(data['blog_posts']) == 3
#     assert 'title' in data['blog_posts'][0]
#     assert 'content' not in data['blog_posts'][0]
#     assert 'author' not in data['blog_posts'][0]



# -*- coding: utf-8 -*-
""" Serafin integration with SQLAlchemy. """
from __future__ import absolute_import, unicode_literals

# 3rd party imports
from . import serialize
from .core import serialize

# local imports
from . import util


def make_serializer(model_base_class):
    """ Create serialize for the given SQLAlchemy base model class.

    This is the class you get by calling SQLAlchemy's declarative_base()
    """
    @serialize.type(model_base_class)
    def serialize_flask_model(obj, spec, ctx):
        """ serafin serializer for ndb models. """
        if spec is True or spec.empty():
            return {}

        data = _serialize_flask_model_fields(obj, spec, ctx)

        props = list(util.iter_public_props(obj, lambda n, v: n in spec))
        data.update({
            name: serialize.raw(value, spec[name], ctx)
            for name, value in props
        })

        return data

    return serialize_flask_model


def _serialize_flask_model_fields(model, spec, ctx):
    """ Serialize SQLAlchemy model class fields. """
    ret = {}

    columns = model.__table__.columns.items()

    for name, _ in columns:
        if name in spec:
            value = getattr(model, name)
            ret[name] = serialize.raw(value, spec[name], ctx)

    return ret



# -*- coding: utf-8 -*-
""" Serafin integration with django ORM. """
from __future__ import absolute_import, unicode_literals

# 3rd party imports
from django.db.models import Model

# local imports
from . import serialize
from . import util


@serialize.type(Model)
def serialize_django_model(obj, spec, ctx):
    """ serafin serializer for django Model instances. """
    if spec is True or spec.empty():
        return {}

    data = serialize_model_fields(obj, spec, ctx)

    props = list(util.iter_public_props(obj, lambda n, v: n in spec))
    data.update({
        name: serialize.raw(value, spec[name], ctx)
        for name, value in props
    })

    return data


def serialize_model_fields(model, fieldspec, context):
    """ Serialize django Model fields

    :param DjangoModel model:
        The model we want to serialize.
    :param Fieldspec fieldspec:
        Defines what fields will be included in the result.
    :param dict context:
        Serialization context. For more information, refer to **serafin**
        documentation.
    :return dict:
        A dictionary containing the serialized data. The layout of the
        dictionary will depend on the fieldspec passed.
    """
    model_fields = [f for f in model._meta.get_fields() if f.name in fieldspec]

    data = {}
    for field in model_fields:
        name = field.name
        value = getattr(model, name)

        if field.is_relation:
            if field.one_to_many:
                rel_spec = fieldspec[name]

                if rel_spec is True or rel_spec.empty():
                    data[name] = []
                else:
                    data[name] = serialize.raw(value.all(), rel_spec, context)
            elif field.many_to_one:
                data[name] = serialize_django_model(
                    value, fieldspec[name], context
                )
            else:
                pass
        else:
            data[name] = value

    return data



# -*- coding: utf-8 -*-
# pylint: disable=missing-docstring,unsubscriptable-object
from __future__ import absolute_import, unicode_literals

# stdlib imports
from unittest import TestCase

# local imports
from serafin.fieldspec import Fieldspec


class FieldspecTest(TestCase):
    def setUp(self):
        self.spec = Fieldspec('field1,field2(sub1,sub2),field3(*)')

    def test_creation(self):
        self.assertEqual(repr(self.spec), '<Fieldspec: field1,field2,field3>')

    def test_contains(self):
        self.assertIn('field1', self.spec)
        self.assertNotIn('field7', self.spec)

    def test_get_subspec(self):
        self.assertEqual(self.spec['field1'], True)

        self.assertEqual(repr(self.spec['field2']), '<Fieldspec: sub1,sub2>')
        self.assertEqual(self.spec['field2']['sub1'], True)
        self.assertIn('sub1', self.spec['field2'])

    def test_return_proper_for_star_fieldspec(self):
        spec = Fieldspec('*')
        self.assertTrue('test' in spec)
        self.assertTrue(spec['test'])

    def test_raises_ValueError_when_invalid_symbols_are_used_in_the_spec(self):
        invalid = ['!@#$%^&[]{}:"\';\\<>,./?']
        for ch in invalid:
            with self.assertRaises(ValueError):
                Fieldspec(ch)

    def test_raises_ValueError_when_unmatched_opening_parenthesis(self):
        with self.assertRaises(ValueError):
            Fieldspec('test(,test2')

        with self.assertRaises(ValueError):
            Fieldspec('test(ttt,adsf(one),test2')

    def test_raises_ValueError_when_unmatched_closing_parenthesis(self):
        with self.assertRaises(ValueError):
            Fieldspec('test(,test2')

        with self.assertRaises(ValueError):
            Fieldspec('test,ttt,adsf(one)),test2')


class TestMerge(TestCase):
    def test_merge_flat_specs(self):
        spec = Fieldspec('field1,field2')
        spec.merge(Fieldspec('field3,field4'))

        self.assertIn('field1', spec)
        self.assertIn('field2', spec)
        self.assertIn('field3', spec)
        self.assertIn('field4', spec)

    def test_merge_simple_compound_specs(self):
        spec = Fieldspec('field1(field2)')
        spec.merge(Fieldspec('field3(field4)'))

        self.assertIn('field1', spec)
        self.assertIn('field3', spec)
        self.assertIn('field2', spec['field1'])
        self.assertIn('field4', spec['field3'])

        self.assertNotIn('field2', spec)
        self.assertNotIn('field4', spec)

    def test_field_in_one_and_excluded_in_other_is_present_in_merge(self):
        merged = Fieldspec('field1,field2,-field3').merge('field2,field3')
        self.assertIn('field1', merged)
        self.assertIn('field2', merged)
        self.assertIn('field3', merged)

    def test_field_in_one_and_excluded_in_other_is_present_in_merge_2(self):
        merged = Fieldspec('field2,field3').merge('field1,field2,-field3')
        self.assertIn('field1', merged)
        self.assertIn('field2', merged)
        self.assertIn('field3', merged)

    def test_field_in_one_and_excluded_in_other_is_present_in_merge_3(self):
        merged = Fieldspec('field2,field3').merge('*,-field3')
        self.assertIn('field1', merged)
        self.assertIn('field2', merged)
        self.assertIn('field3', merged)
        self.assertIn('field99', merged)

    def test_field_excluded_in_both_is_excluded_in_merge(self):
        merged = Fieldspec('field1,field2,-field3').merge('field2,-field3')
        self.assertIn('field1', merged)
        self.assertIn('field2', merged)
        self.assertNotIn('field3', merged)

    def test_field_excluded_in_both_is_excluded_in_merge_2(self):
        merged = Fieldspec('*,-field3').merge('*,-field3')
        self.assertIn('field1', merged)
        self.assertIn('field2', merged)
        self.assertNotIn('field3', merged)
        self.assertIn('field99', merged)


class TestMergeConfict(TestCase):
    def test_merge_simple_fields(self):
        spec = Fieldspec('field1,field2')
        spec.merge(Fieldspec('field2,field3'))

        self.assertEqual(spec.all, False)
        self.assertIn('field1', spec)
        self.assertIn('field2', spec)
        self.assertIn('field3', spec)

    def test_merge_two_fieldspecs(self):
        spec = Fieldspec('field1(field2)')
        spec.merge(Fieldspec('field1(field3)'))

        self.assertEqual(spec.all, False)

        self.assertNotIn('field2', spec)
        self.assertNotIn('field3', spec)

        self.assertIn('field1', spec)
        self.assertIn('field2', spec['field1'])
        self.assertIn('field3', spec['field1'])

    def test_merge_subspec_and_simple_field(self):
        spec = Fieldspec('field1(field2)')
        spec.merge(Fieldspec('field1,field3'))

        self.assertEqual(spec.all, False)

        self.assertIn('field1', spec)
        self.assertIn('field3', spec)
        self.assertIn('field2', spec['field1'])

        self.assertNotIn('field2', spec)

    def test_merge_subspec_and_all(self):
        spec = Fieldspec('field1(field2)')
        spec.merge(Fieldspec('field1(*)'))

        self.assertIn('field1', spec)
        self.assertIn('field99', spec['field1'])

        self.assertNotIn('field2', spec)
        self.assertNotIn('field4', spec)

        self.assertEqual(spec['field1'].all, True)
        self.assertEqual(spec['field1']['asdf'], True)

    def test_merge_simple_field_and_subspec(self):
        spec = Fieldspec('field1,field3')
        spec.merge(Fieldspec('field1(field2)'))

        self.assertEqual(spec.all, False)

        self.assertIn('field1', spec)
        self.assertIn('field3', spec)
        self.assertIn('field2', spec['field1'])

        self.assertNotIn('field2', spec)

    def test_merge_simple_field_and_subspec2(self):
        spec = Fieldspec('field1(field2)')
        spec.merge(Fieldspec('field1,field3'))

        self.assertEqual(spec.all, False)

        self.assertIn('field1', spec)
        self.assertIn('field3', spec)
        self.assertIn('field2', spec['field1'])

        self.assertNotIn('field2', spec)

    def test_merge_all_and_subspec(self):
        spec = Fieldspec('field1(*)')
        spec.merge(Fieldspec('field1(field2)'))

        self.assertIn('field1', spec)
        self.assertIn('field99', spec['field1'])

        self.assertNotIn('field2', spec)
        self.assertNotIn('field4', spec)

        self.assertEqual(spec['field1'].all, True)
        self.assertEqual(spec['field1']['asdf'], True)

    def test_merge_multiple_conflicts(self):
        spec = Fieldspec('field1(*),field2,field3(field31,field32)')
        spec.merge(Fieldspec('field1(field11),field3(field32,field33),field4'))

        self.assertIn('field1', spec)
        self.assertIn('field2', spec)
        self.assertIn('field3', spec)
        self.assertIn('field4', spec)

        self.assertEqual(spec['field1'].all, True)
        self.assertIn('field12', spec['field1'])
        self.assertIn('field11', spec['field1'])
        self.assertIn('asdf', spec['field1'])

        self.assertIn('field31', spec['field3'])
        self.assertIn('field32', spec['field3'])
        self.assertIn('field33', spec['field3'])

    def test_merge_nested_fieldspec(self):
        spec = Fieldspec('field1(field11,field12(field111,field112))')
        other = Fieldspec('field1(field12(field112,field113),field13)')
        spec.merge(other)

        self.assertIn('field1', spec)
        self.assertIn('field11', spec['field1'])
        self.assertIn('field12', spec['field1'])
        self.assertIn('field13', spec['field1'])

        subspec = spec['field1']['field12']
        self.assertIn('field111', subspec)
        self.assertIn('field112', subspec)
        self.assertIn('field113', subspec)


class TestRestrict(TestCase):
    def test_simple_fields(self):
        spec = Fieldspec('field1,field2,field3')
        other = Fieldspec('field2,field3')
        restricted = Fieldspec(spec).restrict(other)

        self.assertNotIn('field1', restricted)
        self.assertIn('field2', restricted)
        self.assertIn('field3', restricted)

    def test_simple_fields_with_exclude(self):
        spec = Fieldspec('field1,field2,field3')
        other = Fieldspec('field2,-field3')
        restricted = Fieldspec(spec).restrict(other)

        self.assertIn('field2', restricted)
        self.assertNotIn('field1', restricted)
        self.assertNotIn('field3', restricted)

    def test_excluded_fields_are_not_exposed_after_restrict(self):
        spec = Fieldspec('field1,field2,-field3')
        other = Fieldspec('field2,field3')
        restricted = Fieldspec(spec).restrict(other)

        self.assertIn('field2', restricted)
        self.assertNotIn('field1', restricted)
        self.assertNotIn('field3', restricted)

    def test_restrict_all(self):
        spec = Fieldspec('*')
        other = Fieldspec('field2,field3')
        restricted = Fieldspec(spec).restrict(other)

        self.assertIn('field2', restricted)
        self.assertIn('field3', restricted)
        self.assertNotIn('field1', restricted)

    def test_restrict_all_with_exclusions(self):
        spec = Fieldspec('*,-field1')
        other = Fieldspec('*,-field4')
        restricted = Fieldspec(spec).restrict(other)

        self.assertIn('field2', restricted)
        self.assertIn('field3', restricted)
        self.assertNotIn('field1', restricted)
        self.assertNotIn('field4', restricted)
        self.assertTrue(restricted.all)

    def test_regression_leaking_all_to_result(self):
        spec = Fieldspec('*,columns(name)')
        other = Fieldspec('id,name')
        restricted = Fieldspec(spec).restrict(other)

        self.assertIn('id', restricted)
        self.assertIn('name', restricted)
        self.assertNotIn('columns', restricted)
        self.assertNotIn('*', restricted)
        self.assertFalse(restricted.all)

    def test_regression_properly_limit_sub_specs(self):
        spec = Fieldspec('*,phone_numbers(number)')
        other = Fieldspec('type,phone_numbers(active,mobile,number)')
        restricted = spec.restrict(other)

        self.assertIn('type', restricted)
        self.assertNotIn('foo', restricted)

        subspec = restricted['phone_numbers']
        self.assertIn('number', subspec)
        self.assertNotIn('active', subspec)
        self.assertNotIn('mobile', subspec)


class TestRestrictWithConflicts(TestCase):
    def test_fieldspec_and_field(self):
        spec = Fieldspec('field1(field11,field12,field13)')
        other = Fieldspec('field1')
        restricted = Fieldspec(spec).restrict(other)

        self.assertEqual(restricted['field1'], True)

    def test_one_level_deep(self):
        spec = Fieldspec('field1(field11,field12,field13)')
        other = Fieldspec('field1(field12,-field13)')
        restricted = Fieldspec(spec).restrict(other)

        subspec = restricted['field1']
        self.assertIn('field12', subspec)
        self.assertNotIn('field11', subspec)
        self.assertNotIn('field13', subspec)

    def test_restrict_by_all_with_exclude(self):
        spec = Fieldspec('field1(field11,field12,field13)')
        other = Fieldspec('field1(*,-field13)')
        restricted = Fieldspec(spec)
        restricted.restrict(other)

        subspec = restricted['field1']
        self.assertIn('field11', subspec)
        self.assertIn('field12', subspec)
        self.assertNotIn('field13', subspec)
